// Anyone is free to copy, modify, use, compile, or distribute this software,
// either in source code form or as a compiled binary, for any non-commercial
// purpose.  For commercial enquiries please contact contact@gimpysoftware.com
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND.
// Gimpy Software does not accept any liability for damage caused by executing
// or modifying this code.
//
// Copyright (c) 2013-2014, Gimpy Software.

using Microsoft.Xna.Framework.Graphics;

namespace LayersBlog
{
    /// <summary>
    /// Interface wrapping a CPU-based XNA graphics effect.
    /// </summary>
    public abstract class CpuLayerEffect : ILayerEffect
    {
        private readonly TextureBuffer m_sourceBuffer = new TextureBuffer();
        private readonly TextureBuffer m_destBuffer = new TextureBuffer();

        public void Apply(SpriteBatch spriteBatch, Texture2D source, Texture2D destination)
        {
            // Apply effect from source to destination buffer.
            m_sourceBuffer.SetSize(source.Width, source.Height);
            m_destBuffer.SetSize(source.Width, source.Height);

            source.GetData(m_sourceBuffer.Buffer);
            ModifyTexture(m_sourceBuffer, m_destBuffer, source.Width, source.Height);

            spriteBatch.GraphicsDevice.SetRenderTarget(null);
            destination.SetData(m_destBuffer.Buffer);
        }

        protected abstract void ModifyTexture(TextureBuffer sourceBuffer, TextureBuffer destBuffer, int width, int height);
    }
}