// Anyone is free to copy, modify, use, compile, or distribute this software,
// either in source code form or as a compiled binary, for any non-commercial
// purpose.  For commercial enquiries please contact contact@gimpysoftware.com
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND.
// Gimpy Software does not accept any liability for damage caused by executing
// or modifying this code.
//
// Copyright (c) 2013-2014, Gimpy Software.

using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;

namespace LayersBlog
{
    public class LayersDemo : Game
    {
        private readonly GraphicsDeviceManager m_graphics;
        private SpriteBatch m_spriteBatch;
        private Layers m_rootLayers;
        private readonly Logger m_logger;

        public LayersDemo()
        {
            m_graphics = new GraphicsDeviceManager(this)
            {
                PreferMultiSampling = true,
                PreferredBackBufferWidth = 640,
                PreferredBackBufferHeight = 400
            };

            m_logger = new Logger();
            m_logger.LogEnvironment();

            Content.RootDirectory = "Content";
        }

        protected override void LoadContent()
        {
            m_spriteBatch = new SpriteBatch(GraphicsDevice);

            var speccyFont = Content.Load<SpriteFont>("Speccy8x8");

            // Define each layer.
            var textureLayer = new TextureLayer((Content.Load<Texture2D>("london")));
            var statsLayer = new StatsLayer(m_graphics.GraphicsDevice, m_graphics.PreferredBackBufferWidth, m_graphics.PreferredBackBufferHeight, speccyFont);

            // Create a Layers object, and define the order of the layers to display.
            m_rootLayers = new Layers(m_graphics.GraphicsDevice, m_graphics.PreferredBackBufferWidth, m_graphics.PreferredBackBufferHeight);
            m_rootLayers.Add(textureLayer);
            m_rootLayers.Add(new LoggerLayer(m_graphics.GraphicsDevice, m_graphics.PreferredBackBufferWidth, m_graphics.PreferredBackBufferHeight, speccyFont, m_logger));

            WriteSomeLogMessages();
        }

        private void WriteSomeLogMessages()
        {
            m_logger.Log("Content loaded.");
            m_logger.Log("Message 1\nMessage 2\nMessage 3");
            for (int i = 0; i < 100; i++)
                m_logger.Log("Doing something: " + i);

            m_logger.Log("Last message.");
        }

        protected override void Update(GameTime gameTime)
        {
            if (Keyboard.GetState().IsKeyDown(Keys.Escape))
            {
                Exit();
                return;
            }

            m_rootLayers.Update(gameTime);

            base.Update(gameTime);
        }

        protected override void Draw(GameTime gameTime)
        {
            GraphicsDevice.Clear(Color.CornflowerBlue);

            m_rootLayers.Draw(m_spriteBatch);

            base.Draw(gameTime);
        }
    }
}
